@@ -161,6 +161,13 @@ class FormatDecisionState(object):
       # Split before the closing bracket if we can.
       return current.node_split_penalty != split_penalty.UNBREAKABLE
 
+    # Prevent splitting before the first argument in compound statements
+    # with the exception of function declarations.
+    if (style.Get('SPLIT_BEFORE_FIRST_ARGUMENT') and
+        self.line.first.value != 'def' and
+        self.line.first.value in _COMPOUND_STMTS):
+      return False
+
     ###########################################################################
     # List Splitting
     if (style.Get('DEDENT_CLOSING_BRACKETS') or
@@ -189,6 +196,38 @@ class FormatDecisionState(object):
           # Split before and dedent the closing bracket.
           return self.stack[-1].split_before_closing_bracket
 
+    if (current.is_name or current.is_string) and previous.value == ',':
+      # If the list has function calls in it and the full list itself cannot
+      # fit on the line, then we want to split. Otherwise, we'll get something
+      # like this:
+      #
+      #     X = [
+      #         Bar(xxx='some string',
+      #             yyy='another long string',
+      #             zzz='a third long string'), Bar(
+      #                 xxx='some string',
+      #                 yyy='another long string',
+      #                 zzz='a third long string')
+      #     ]
+      #
+      # or when a string formatting syntax.
+      func_call_or_string_format = False
+      if current.is_name:
+        tok = current.next_token
+        while tok and (tok.is_name or tok.value == '.'):
+          tok = tok.next_token
+        func_call_or_string_format = tok and tok.value == '('
+      elif current.is_string:
+        tok = current.next_token
+        while tok and tok.is_string:
+          tok = tok.next_token
+        func_call_or_string_format = tok and tok.value == '%'
+      if func_call_or_string_format:
+        open_bracket = unwrapped_line.IsSurroundedByBrackets(current)
+        if open_bracket and open_bracket.value in '[{':
+          if not self._FitsOnLine(open_bracket, open_bracket.matching_bracket):
+            return True
+
     ###########################################################################
     # Dict/Set Splitting
     if (style.Get('EACH_DICT_ENTRY_ON_SEPARATE_LINE') and
@@ -251,7 +290,7 @@ class FormatDecisionState(object):
             return False
 
           column = self.column - self.stack[-1].last_space
-          return column >= style.Get('CONTINUATION_INDENT_WIDTH')
+          return column > style.Get('CONTINUATION_INDENT_WIDTH')
 
         opening = _GetOpeningBracket(current)
         if opening:
@@ -268,20 +307,23 @@ class FormatDecisionState(object):
           if opening.matching_bracket.previous_token.value == ',':
             return True
 
-    if current.is_name and previous.value == ',':
+    if ((current.is_name or current.value in {'*', '**'}) and
+        previous.value == ','):
       # If we have a function call within an argument list and it won't fit on
       # the remaining line, but it will fit on a line by itself, then go ahead
       # and split before the call.
       opening = _GetOpeningBracket(current)
       if (opening and opening.value == '(' and opening.previous_token and
-          opening.previous_token.is_name):
+          (opening.previous_token.is_name or
+           opening.previous_token.value in {'*', '**'})):
         is_func_call = False
         token = current
         while token:
           if token.value == '(':
             is_func_call = True
             break
-          if not token.is_name and token.value != '.':
+          if (not (token.is_name or token.value in {'*', '**'}) and
+              token.value != '.'):
             break
           token = token.next_token
 
@@ -459,7 +501,12 @@ class FormatDecisionState(object):
          previous.previous_token.OpensScope())):
       self.stack[-1].closing_scope_indent = max(
           0, self.stack[-1].indent - style.Get('CONTINUATION_INDENT_WIDTH'))
-      self.stack[-1].split_before_closing_bracket = True
+
+      split_before_closing_bracket = True
+      if style.Get('COALESCE_BRACKETS'):
+        split_before_closing_bracket = False
+
+      self.stack[-1].split_before_closing_bracket = split_before_closing_bracket
 
     # Calculate the split penalty.
     penalty = current.split_penalty
