@@ -121,8 +121,8 @@ class FormatToken(object):
     """
     indent_char = '\t' if style.Get('USE_TABS') else ' '
     token_indent_char = indent_char if newlines_before > 0 else ' '
-    indent_before = (
-        indent_char * indent_level * style.Get('INDENT_WIDTH') + token_indent_char * spaces)
+    indent_before = (indent_char * indent_level * style.Get('INDENT_WIDTH') +
+                     token_indent_char * spaces)
 
     if self.is_comment:
       comment_lines = [s.lstrip() for s in self.value.splitlines()]
