@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@ import difflib
 import re
 import sys
 
+from lib2to3.pgen2 import parse
 from lib2to3.pgen2 import tokenize
 
 from yapf.yapflib import blank_line_calculator
@@ -80,7 +81,7 @@ def FormatFile(filename,
   if in_place and print_diff:
     raise ValueError('Cannot pass both in_place and print_diff.')
 
-  original_source, encoding = ReadFile(filename, logger)
+  original_source, newline, encoding = ReadFile(filename, logger)
   reformatted_source, changed = FormatCode(
       original_source,
       style_config=style_config,
@@ -88,6 +89,9 @@ def FormatFile(filename,
       lines=lines,
       print_diff=print_diff,
       verify=verify)
+  if reformatted_source.rstrip('\n'):
+    lines = reformatted_source.rstrip('\n').split('\n')
+    reformatted_source = newline.join(line for line in lines) + newline
   if in_place:
     if original_source and original_source != reformatted_source:
       file_resources.WriteReformattedCode(filename, reformatted_source,
@@ -120,7 +124,11 @@ def FormatCode(unformatted_source,
   style.SetGlobalStyle(style.CreateStyleFromConfig(style_config))
   if not unformatted_source.endswith('\n'):
     unformatted_source += '\n'
-  tree = pytree_utils.ParseCodeToTree(unformatted_source)
+
+  try:
+    tree = pytree_utils.ParseCodeToTree(unformatted_source)
+  except parse.ParseError as e:
+    raise parse.ParseError(filename + ': ' + e.message)
 
   # Run passes on the tree, modifying it in place.
   comment_splicer.SpliceComments(tree)
@@ -184,18 +192,22 @@ def ReadFile(filename, logger=None):
     raise
 
   try:
+    # Preserves line endings.
     with py3compat.open_with_encoding(
-        filename, mode='r', encoding=encoding) as fd:
-      source = fd.read()
-    return source, encoding
+        filename, mode='r', encoding=encoding, newline='') as fd:
+      lines = fd.readlines()
+
+    line_ending = file_resources.LineEnding(lines)
+    source = '\n'.join(line.rstrip('\r\n') for line in lines) + '\n'
+    return source, line_ending, encoding
   except IOError as err:  # pragma: no cover
     if logger:
       logger(err)
     raise
 
 
-DISABLE_PATTERN = r'^#+ +yapf: *disable$'
-ENABLE_PATTERN = r'^#+ +yapf: *enable$'
+DISABLE_PATTERN = r'^#.*\byapf:\s*disable\b'
+ENABLE_PATTERN = r'^#.*\byapf:\s*enable\b'
 
 
 def _MarkLinesToFormat(uwlines, lines):
@@ -204,25 +216,48 @@ def _MarkLinesToFormat(uwlines, lines):
     for uwline in uwlines:
       uwline.disable = True
 
-    for start, end in sorted(lines):
-      for uwline in uwlines:
-        if uwline.lineno > end:
-          break
-        if uwline.lineno >= start:
-          uwline.disable = False
-        elif uwline.last.lineno >= start:
-          uwline.disable = False
+    # Sort and combine overlapping ranges.
+    lines = sorted(lines)
+    line_ranges = [lines[0]] if len(lines[0]) else []
+    index = 1
+    while index < len(lines):
+      current = line_ranges[-1]
+      if lines[index][0] <= current[1]:
+        # The ranges overlap, so combine them.
+        line_ranges[-1] = (current[0], max(lines[index][1], current[1]))
+      else:
+        line_ranges.append(lines[index])
+      index += 1
 
+    # Mark lines to format as not being disabled.
+    index = 0
+    for start, end in sorted(line_ranges):
+      while index < len(uwlines) and uwlines[index].last.lineno < start:
+        index += 1
+      if index >= len(uwlines):
+        break
+
+      while index < len(uwlines):
+        if uwlines[index].lineno > end:
+          break
+        if (uwlines[index].lineno >= start or
+            uwlines[index].last.lineno >= start):
+          uwlines[index].disable = False
+        index += 1
+
+  # Now go through the lines and disable any lines explicitly marked as
+  # disabled.
   index = 0
   while index < len(uwlines):
     uwline = uwlines[index]
     if uwline.is_comment:
       if _DisableYAPF(uwline.first.value.strip()):
+        index += 1
         while index < len(uwlines):
           uwline = uwlines[index]
-          uwline.disable = True
           if uwline.is_comment and _EnableYAPF(uwline.first.value.strip()):
             break
+          uwline.disable = True
           index += 1
     elif re.search(DISABLE_PATTERN, uwline.last.value.strip(), re.IGNORECASE):
       uwline.disable = True
@@ -230,17 +265,15 @@ def _MarkLinesToFormat(uwlines, lines):
 
 
 def _DisableYAPF(line):
-  return (re.search(DISABLE_PATTERN, line.split('\n')[0].strip(),
-                    re.IGNORECASE) or re.search(DISABLE_PATTERN,
-                                                line.split('\n')[-1].strip(),
-                                                re.IGNORECASE))
+  return (
+      re.search(DISABLE_PATTERN, line.split('\n')[0].strip(), re.IGNORECASE) or
+      re.search(DISABLE_PATTERN, line.split('\n')[-1].strip(), re.IGNORECASE))
 
 
 def _EnableYAPF(line):
-  return (re.search(ENABLE_PATTERN, line.split('\n')[0].strip(),
-                    re.IGNORECASE) or re.search(ENABLE_PATTERN,
-                                                line.split('\n')[-1].strip(),
-                                                re.IGNORECASE))
+  return (
+      re.search(ENABLE_PATTERN, line.split('\n')[0].strip(), re.IGNORECASE) or
+      re.search(ENABLE_PATTERN, line.split('\n')[-1].strip(), re.IGNORECASE))
 
 
 def _GetUnifiedDiff(before, after, filename='code'):
